@@ -11,10 +11,10 @@ use crate::{cvt, cvt_p};
 /// A type used to derive a shared secret between two keys.
 pub struct Deriver<'a>(*mut ffi::EVP_PKEY_CTX, PhantomData<&'a ()>);
 
-unsafe impl<'a> Sync for Deriver<'a> {}
-unsafe impl<'a> Send for Deriver<'a> {}
+unsafe impl Sync for Deriver<'_> {}
+unsafe impl Send for Deriver<'_> {}
 
-impl<'a> Drop for Deriver<'a> {
+impl Drop for Deriver<'_> {
     fn drop(&mut self) {
         unsafe {
             ffi::EVP_PKEY_CTX_free(self.0);
