@@ -102,7 +102,7 @@ foreign_type_and_impl_send_sync! {
     /// Prime fields use the formula `y^2 mod p = x^3 + ax + b mod p`.  Binary
     /// fields use the formula `y^2 + xy = x^3 + ax^2 + b`.  Named curves have
     /// assured security.  To prevent accidental vulnerabilities, they should
-    /// be prefered.
+    /// be preferred.
     ///
     /// [wiki]: https://wiki.openssl.org/index.php/Command_Line_Elliptic_Curve_Operations
     /// [`Nid`]: ../nid/index.html
@@ -174,6 +174,7 @@ impl EcGroupRef {
     /// OpenSSL documentation at [`EC_GROUP_get_degree`]
     ///
     /// [`EC_GROUP_get_degree`]: https://www.openssl.org/docs/man1.1.0/crypto/EC_GROUP_get_degree.html
+    #[allow(clippy::unnecessary_cast)]
     pub fn degree(&self) -> u32 {
         unsafe { ffi::EC_GROUP_get_degree(self.as_ptr()) as u32 }
     }
@@ -621,7 +622,7 @@ where
 
     /// Checks the key for validity.
     ///
-    /// OpenSSL documenation at [`EC_KEY_check_key`]
+    /// OpenSSL documentation at [`EC_KEY_check_key`]
     ///
     /// [`EC_KEY_check_key`]: https://www.openssl.org/docs/man1.1.0/crypto/EC_KEY_check_key.html
     pub fn check_key(&self) -> Result<(), ErrorStack> {
@@ -647,7 +648,7 @@ impl EcKey<Params> {
     /// It will not have an associated public or private key. This kind of key is primarily useful
     /// to be provided to the `set_tmp_ecdh` methods on `Ssl` and `SslContextBuilder`.
     ///
-    /// OpenSSL documenation at [`EC_KEY_new_by_curve_name`]
+    /// OpenSSL documentation at [`EC_KEY_new_by_curve_name`]
     ///
     /// [`EC_KEY_new_by_curve_name`]: https://www.openssl.org/docs/man1.1.0/crypto/EC_KEY_new_by_curve_name.html
     pub fn from_curve_name(nid: Nid) -> Result<EcKey<Params>, ErrorStack> {
